@@ -1,5 +1,5 @@
 (*  Wyrd -- a curses-based front-end for Remind
- *  Copyright (C) 2005, 2006, 2007 Paul Pelzl
+ *  Copyright (C) 2005, 2006, 2007, 2008, 2010, 2011-2013 Paul Pelzl
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, Version 2,
@@ -14,8 +14,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  Please send bug reports, patches, etc. to Paul Pelzl at 
- *  <pelzlpj@eecs.umich.edu>.
+ *  Bug reports can be entered at http://bugs.launchpad.net/wyrd .
+ *  For anything else, feel free to contact Paul Pelzl at <pelzlpj@gmail.com>.
  *)
 
 (* interface_main.ml
@@ -140,6 +140,42 @@ let resize_subwins iface =
    new_scr
 
 
+(* Drop curses mode, invoke a function, and restore curses mode. *)
+let drop_curses_across (f : 'a -> 'b)  (x : 'a) : 'b =
+   def_prog_mode ();
+   endwin ();
+   let result = f x in
+   reset_prog_mode ();
+   let _ = curs_set 0 in
+   result
+
+
+(* Invoke an external process as Unix.system, but loop on EINTR while waiting for process
+ * completion.  This is a workaround for the fact that SIGWINCH may be received while curses mode is
+ * deactivated when we launch an external editor. *)
+let system_retry_eintr (command : string) : Unix.process_status =
+  match Unix.fork () with
+  | 0 ->
+      begin try
+         Unix.execv "/bin/sh" [| "/bin/sh"; "-c"; command |]
+      with _ ->
+         exit 127
+      end
+  | pid -> 
+      let rec loop_eintr () =
+         try
+            snd (Unix.waitpid [] pid)
+         with Unix.Unix_error (Unix.EINTR, _, _) ->
+            loop_eintr ()
+      in
+      loop_eintr ()
+
+     
+
+let drop_curses_system (command : string) : Unix.process_status =
+   drop_curses_across system_retry_eintr command
+
+
 (* Parse a natural language event description and append
  * the result to the specified reminders file. *)
 let append_quick_event event_spec remfile =
@@ -233,9 +269,9 @@ let handle_resize (iface : interface_state_t) reminders =
        * paint an error message. *)
       let w = newwin 0 0 0 0 in
       erase ();
-      assert (mvwaddstr w 0 0 s);
-      assert (wnoutrefresh w);
-      assert (doupdate ());
+      let _ = mvwaddstr w 0 0 s in
+      let _ = wnoutrefresh w in
+      let _ = doupdate () in
       ({iface with resize_failed_win = Some w}, reminders)
    end
       
@@ -634,15 +670,7 @@ let handle_new_reminder (iface : interface_state_t) reminders rem_type
          Str.global_replace filename_sub remfile
             !Rcfile.edit_new_command
       in
-      def_prog_mode ();
-      endwin ();
-      let editor_process_status = Unix.system command in 
-      reset_prog_mode ();
-      begin try
-         assert (curs_set 0)
-      with _ ->
-         ()
-      end;
+      let editor_process_status = drop_curses_system command in
       let r = Remind.create_three_month iface.top_timestamp in
       (* if the untimed list has been altered, change the focus to
        * the first element of the list *)
@@ -997,17 +1025,10 @@ let handle_view_reminders (iface : interface_state_t) reminders trigger_all =
       else
          !Rcfile.remind_command ^ " -q -g "
    in
-   let command = partial_command ^ !Rcfile.reminders_file ^ " " ^
-   rem_date_str ^ " | less -c" in
-   def_prog_mode ();
-   endwin ();
-   let _ = Unix.system command in 
-   reset_prog_mode ();
-   begin try
-      assert (curs_set 0)
-   with _ ->
-      ()
-   end;
+   let command =
+      partial_command ^ !Rcfile.reminders_file ^ " " ^ rem_date_str ^ " | less -c"
+   in
+   let _ = drop_curses_system command in
    handle_refresh iface reminders
 
 
@@ -1033,15 +1054,7 @@ let handle_view_calendar (iface : interface_state_t) reminders week_only =
       partial_command ^ time_option ^ weekday_option ^ 
       !Rcfile.reminders_file ^ " " ^ rem_date_str ^ " | less -c" 
    in
-   def_prog_mode ();
-   endwin ();
-   let _ = Unix.system command in 
-   reset_prog_mode ();
-   begin try
-      assert (curs_set 0)
-   with _ ->
-      ()
-   end;
+   let _ = drop_curses_system command in
    handle_refresh iface reminders
 
 
@@ -1178,12 +1191,11 @@ let handle_edit (iface : interface_state_t) reminders =
    in
    begin match fl with
    |None ->
+      let (main_remfile, _) = Remind.get_all_remfiles () in
       if iface.selected_side = Left then
-         handle_new_reminder iface reminders Timed
-         (Utility.expand_file !Rcfile.reminders_file)
+         handle_new_reminder iface reminders Timed main_remfile
       else
-         handle_new_reminder iface reminders Untimed
-         (Utility.expand_file !Rcfile.reminders_file)
+         handle_new_reminder iface reminders Untimed main_remfile
    |Some (filename, line_num, msg) -> 
       let filename_sub = Str.regexp "%file%" in
       let lineno_sub   = Str.regexp "%line%" in
@@ -1191,15 +1203,7 @@ let handle_edit (iface : interface_state_t) reminders =
          Str.global_replace filename_sub filename !Rcfile.edit_old_command
       in
       let command = Str.global_replace lineno_sub line_num command_partial in
-      def_prog_mode ();
-      endwin ();
-      let _ = Unix.system command in 
-      reset_prog_mode ();
-      begin try
-         assert (curs_set 0)
-      with _ ->
-         ()
-      end;
+      let _ = drop_curses_system command in
       let r = Remind.create_three_month iface.top_timestamp in
       (* if the untimed list has been altered, change the focus
        * to the first element *)
@@ -1224,15 +1228,7 @@ let handle_edit_any (iface : interface_state_t) reminders remfile =
    let command = 
       Str.global_replace filename_sub remfile !Rcfile.edit_any_command
    in
-   def_prog_mode ();
-   endwin ();
-   let _ = Unix.system command in 
-   reset_prog_mode ();
-   begin try
-      assert (curs_set 0)
-   with _ ->
-      ()
-   end;
+   let _ = drop_curses_system command in
    let r = Remind.create_three_month iface.top_timestamp in
    (* if the untimed list has been altered, change the focus
     * to the first element *)
@@ -1409,15 +1405,7 @@ let handle_paste_reminder (iface : interface_state_t) reminders remfile =
       Str.global_replace filename_sub remfile
          !Rcfile.edit_new_command
    in
-   def_prog_mode ();
-   endwin ();
-   let _ = Unix.system command in 
-   reset_prog_mode ();
-   begin try
-      assert (curs_set 0)
-   with _ ->
-      ()
-   end;
+   let _ = drop_curses_system command in
    let r = Remind.create_three_month iface.top_timestamp in
    (* if the untimed list has been altered, change the focus to
     * the first element of the list *)
@@ -1678,7 +1666,7 @@ let handle_keypress_normal key (iface : interface_state_t) reminders =
       |Rcfile.Edit ->
          handle_edit iface reminders
       |Rcfile.EditAny ->
-         let all_remfiles = Remind.get_included_remfiles () in
+         let (_, all_remfiles) = Remind.get_all_remfiles () in
          let selected_remfile = 
             (* if there's only one remfile, jump right in, otherwise
              * pop up a selection dialog *)
@@ -1700,8 +1688,8 @@ let handle_keypress_normal key (iface : interface_state_t) reminders =
             assert (doupdate ());
             (iface, reminders)
          end else begin
-            handle_paste_reminder iface reminders 
-            (Utility.expand_file !Rcfile.reminders_file)
+            let (main_remfile, _) = Remind.get_all_remfiles () in
+            handle_paste_reminder iface reminders main_remfile
          end
       |Rcfile.PasteReminderDialog ->
          if iface.rem_buffer = "" then begin
@@ -1710,7 +1698,7 @@ let handle_keypress_normal key (iface : interface_state_t) reminders =
             assert (doupdate ());
             (iface, reminders)
          end else begin
-            let all_remfiles = Remind.get_included_remfiles () in
+            let (_, all_remfiles) = Remind.get_all_remfiles () in
             let selected_remfile = 
                (* if there's only one remfile, jump right in, otherwise
                 * pop up a selection dialog *)
@@ -1729,30 +1717,30 @@ let handle_keypress_normal key (iface : interface_state_t) reminders =
       |Rcfile.QuickEvent ->
          handle_begin_quick_event iface reminders
       |Rcfile.NewTimed ->
-         handle_new_reminder iface reminders Timed
-         (Utility.expand_file !Rcfile.reminders_file)
+         let (main_remfile, _) = Remind.get_all_remfiles () in
+         handle_new_reminder iface reminders Timed main_remfile
       |Rcfile.NewTimedDialog ->
          let remfile = 
-            do_selection_dialog iface "Choose a reminders file"
-            (Remind.get_included_remfiles ())
+            let (_, all_remfiles) = Remind.get_all_remfiles () in
+            do_selection_dialog iface "Choose a reminders file" all_remfiles
          in
          handle_new_reminder iface reminders Timed remfile
       |Rcfile.NewUntimed ->
-         handle_new_reminder iface reminders Untimed
-         (Utility.expand_file !Rcfile.reminders_file)
+         let (main_remfile, _) = Remind.get_all_remfiles () in
+         handle_new_reminder iface reminders Untimed main_remfile
       |Rcfile.NewUntimedDialog ->
          let remfile = 
-            do_selection_dialog iface "Choose a reminders file"
-            (Remind.get_included_remfiles ())
+            let (_, all_remfiles) = Remind.get_all_remfiles () in
+            do_selection_dialog iface "Choose a reminders file" all_remfiles
          in
          handle_new_reminder iface reminders Untimed remfile
       |Rcfile.NewGenReminder x ->
-         handle_new_reminder iface reminders (General x)
-         (Utility.expand_file !Rcfile.reminders_file)
+         let (main_remfile, _) = Remind.get_all_remfiles () in
+         handle_new_reminder iface reminders (General x) main_remfile
       |Rcfile.NewGenReminderDialog x ->
          let remfile = 
-            do_selection_dialog iface "Choose a reminders file"
-            (Remind.get_included_remfiles ())
+            let (_, all_remfiles) = Remind.get_all_remfiles () in
+            do_selection_dialog iface "Choose a reminders file" all_remfiles
          in
          handle_new_reminder iface reminders (General x) remfile
       |Rcfile.SearchNext ->
@@ -1784,6 +1772,7 @@ let handle_keypress_normal key (iface : interface_state_t) reminders =
             ()
          end;
          let i = draw_msg iface in
+         let reminders = Remind.create_three_month i.top_timestamp in
          handle_refresh i reminders
       |Rcfile.Quit ->
          let new_iface = {iface with run_wyrd = false} in
@@ -1839,8 +1828,8 @@ let handle_keypress key (iface : interface_state_t) reminders =
                end
             |ExtendedQuick ->
                begin try
-                  handle_quick_event iface reminders
-                  (Utility.expand_file !Rcfile.reminders_file)
+                  let (main_remfile, _) = Remind.get_all_remfiles () in
+                  handle_quick_event iface reminders main_remfile
                with Failure err ->
                   let new_iface = {
                      iface with extended_input = "";
@@ -1913,8 +1902,10 @@ let handle_keypress key (iface : interface_state_t) reminders =
             end
          |_ ->
             begin try
-               (* only printable characters are accepted for search strings and quick events *)
-               if key >= 32 && key <= 126 then begin
+               (* Only printable characters are accepted for search strings and quick events.  "Printable" 
+                * is defined as either "printable ASCII" or "non-ASCII UTF-8".  Hopefully this definition
+                * will satisfy ncurses. *)
+               if (Curses_config.wide_ncurses && key >= 0x80) || (key >= 32 && key <= 126) then begin
                   let c = char_of_int key in
                   let new_iface = {
                      iface with extended_input = iface.extended_input ^ (String.make 1 c)
@@ -1935,7 +1926,7 @@ let handle_keypress key (iface : interface_state_t) reminders =
                      |ExtendedGoto   -> "logical error in generating extended entry string"
                   in
                   failwith err_msg
-            with Failure _ ->
+            with Failure _ | Invalid_argument "char_of_int" ->
                let _ = beep () in
                (iface, reminders)
             end
@@ -1951,14 +1942,17 @@ let rec do_main_loop (iface : interface_state_t) reminders last_update =
          reminders.Remind.remind_error ^ "\"") false
       else
          ();
-      (* refresh the msg window (which contains a clock)
-       * every wgetch timeout cycle *)
-      let iface = draw_msg iface in
-      begin match iface.resize_failed_win with
-      | None   -> assert (doupdate ())
-      | Some w -> ()
-      end;
-      let key = wgetch iface.scr.help_win in
+      let (iface, wgetch_target) =
+        match iface.resize_failed_win with
+        | None ->
+            (* refresh the msg window (which contains a clock) every wgetch timeout cycle *)
+            let iface = draw_msg iface in
+            assert (doupdate ());
+            (iface, iface.scr.help_win)
+        | Some w ->
+            (iface, w)
+      in
+      let key = wgetch wgetch_target in
       let new_iface, new_reminders = 
          (* key = -1 is ncurses wgetch timeout error *)
          if key <> ~- 1 then
@@ -1975,21 +1969,25 @@ let rec do_main_loop (iface : interface_state_t) reminders last_update =
          else
             (iface, reminders)
       in
+      let remfile_mtimes = Remind.get_remfile_mtimes () in
       let curr_time = Unix.time () in
-      (* poll remind(1) every 5 minutes and update display *)
-      if curr_time -. last_update > 300.0 then begin
+      (* if stat() indicates that remfiles were updated, or if five minutes has passed,
+       * then poll remind(1) and update the display *)
+      if (remfile_mtimes <> new_iface.remfile_mtimes || curr_time -. last_update > 300.0) &&
+            iface.resize_failed_win = None then begin
          let r = Remind.create_three_month new_iface.top_timestamp in
          (* if the untimed list has been altered, change the focus to
           * the timed window *)
          let new_iface =
             if List.length r.Remind.curr_untimed <> 
-               List.length reminders.Remind.curr_untimed then {
+                  List.length reminders.Remind.curr_untimed then {
                new_iface with selected_side = Left;
                               top_untimed = 0;
-                              right_selection = 1
+                              right_selection = 1;
+                              remfile_mtimes = remfile_mtimes
                }
             else
-               new_iface
+               {new_iface with remfile_mtimes = remfile_mtimes}
          in
          let (iface2, reminders2) = handle_refresh new_iface r in
          do_main_loop iface2 reminders2 curr_time
@@ -2016,6 +2014,7 @@ let run (iface : interface_state_t) =
    set_bkgd iface.scr.msg_win Rcfile.Description; 
    scrollok iface.scr.timed_win true;
    let reminders = Remind.create_three_month (iface.top_timestamp) in
+   let iface = {iface with remfile_mtimes = Remind.get_remfile_mtimes ()} in
    assert (keypad iface.scr.help_win true);
    draw_help iface;
    draw_date_strip iface;
